#region usings
using System.Collections.Generic;
using System.ComponentModel.Composition;
using VVVV.PluginInterfaces.V2;
using VVVV.Core.Logging;
using DAP.CompGeom;
using VVVV.Utils.VMath;

#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "VoronoiComplex", Category = "2d.Voronoi", Help = "Returns voronoi information", Tags = "Voronoi, Delaunay")]
	#endregion PluginInfo
	public class VoronoiComplex : IPluginEvaluate
	{
		#region fields & pins
		[Input("Points", DefaultValue = 0.0)]
		public IDiffSpread<Vector2D> Points;

		[Output("Voronoi Complex")]
		public ISpread<WingedEdge<FortunePoly, FortuneEdge, FortuneVertex>> ComplxOut;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int spreadMax)
		{
			if (!Points.IsChanged)
			{
				return;
			}
			ComplxOut.SliceCount = 1;
			var points = new List<PointD>(spreadMax);

			for (var i = 0; i < spreadMax; i++)
			{
				points.Add(new PointD(Points[i].x, Points[i].y));
			}
			ComplxOut[0] = Fortune.ComputeVoronoi(points);
		}
	}
}
